#!/usr/bin/env ruby

#
# Combine voice and face recognition times
#
class CombineRecognitions

  # add an offset in seconds for the voice cuts at start and end
  OFFSET_VOICE_START = 1
  OFFSET_VOICE_END = 2

  # time between moderations >= 13 seconds
  MINIMAL_TIME_BETWEEN_MODERATIONS_IN_SECONDS = 13

  attr_reader :filename_face, :filename_voice, :faceTimeRanges, :voiceTimeRanges

  def getTimeRanges(filename, offsetStart = 0, offsetEnd = 0)
    timeRanges = Array.new
    lineCounter = 0

    File.open(filename).each do |line|
      startSeconds=(line[/.*Final: (.*) - .*/,1]).to_i - offsetStart
      endSeconds=(line[/.*- (.*) =.*/,1]).to_i + offsetEnd

      startSeconds = 0 if startSeconds < 0

      # print "#{lineCounter}: #{startSeconds} #{endSeconds}\n"
      timeRanges[lineCounter] = (startSeconds..endSeconds).to_a

      lineCounter += 1
    end

    return timeRanges.flatten
  end

  def initialize(filename_face, filename_voice)
    @filename_face = filename_face
    @filename_voice = filename_voice

    @faceTimeRanges = getTimeRanges(@filename_face)
    @voiceTimeRanges = getTimeRanges(@filename_voice, OFFSET_VOICE_START, OFFSET_VOICE_END)
  end

  def findIntersections
    # print "Face seconds : #{@faceTimeRanges}\n\n"
    # print "Voice seconds: #{@voiceTimeRanges}\n\n"
    result = @faceTimeRanges & @voiceTimeRanges

    print "Intersections: #{result}\n\n" if !result.empty?
    print "Number of overlapping seconds:\n#{result.size}\n"
  end

  def combineTimes
    result = (@faceTimeRanges + @voiceTimeRanges).uniq.sort

    # print "Combinations: #{result}\n"
    changedResult = result

    previous = result[0] - 1

    result.each do |number|
      if previous + 1 != number
        if previous + MINIMAL_TIME_BETWEEN_MODERATIONS_IN_SECONDS >= number
          # print "#{previous + 1} #{number - 1}\n"
          changedResult = changedResult + ((previous + 1)..(number - 1)).to_a
        end
      end
      previous = number
    end

    result = changedResult.sort
    # print "Combinations: #{result}\n"

    print "#{result[0]}"
    previous = result[0] - 1

    result.each do |number|
      # check end
      if previous + 1 != number
        print "-#{previous}\n#{number}"
      end
      previous = number
    end

    print "-#{result[-1]}\n"

  end

end

myCombineRecognitions = CombineRecognitions.new(ARGV[0], ARGV[1])

if ARGV.length == 2
  myCombineRecognitions.findIntersections
else
  myCombineRecognitions.combineTimes
end
