#!/bin/bash

if [ $# -ne 1 ];then
  echo "Please add the filename of the video as parameter"
  exit 1
fi

SCRIPT_DIRECTORY="$( cd "$( dirname "${BASH_SOURCE[0]}" )" && pwd )"

video="$1"
video_directory=$(dirname "$video")
file_base_name=$(basename "$video")
file_base_name_without_extenstion=$(basename "$video" | cut -d. -f1)

voice_directory="${video_directory}/${file_base_name_without_extenstion}/voice_recognition/"

if [ ! -d "$voice_directory" ];then
  mkdir -p "$voice_directory"
fi

echo "Creating voice id files. This can take several minutes. Please be patient."

cd "$voice_directory"
if [ ! -h "$file_base_name" ];then
  ln -s ../../"$file_base_name" .
fi

# echo "$file_base_name_without_extenstion"

function runVoiceID {

  # echo "=== $PWD"

  # check whether voice id was already run
  if [ -e "${file_base_name_without_extenstion}" ];then
    if [ "$(ls -A ${file_base_name_without_extenstion})" != "" ];then
      echo "Directory for voice id already filled. Delete it manually if you want to run voice id again."
      return
    fi
  fi

  startSeconds=$(date +"%s")
  echo "Start at:"
  date

  vid -i "$file_base_name" -k

  endSeconds=$(date +"%s")

  let duration=endSeconds-startSeconds
  let durationMinutes=(duration+59)/60

  echo "End at:"
  date
  echo "VoiceID finished after: $duration seconds  resp.  $durationMinutes minutes" | tee time.log

}

runVoiceID

cd "${file_base_name_without_extenstion}"
ls -1 */*wav | sed 's|^\([^/]*\)/.*_\(0[0-9]*\)\.\(0[0-9]*\)\.wav|\2 \3 \1|g' | sort > voices.txt

ruby "$SCRIPT_DIRECTORY"/parseVoiceIdentifiers.rb voices.txt > parsed_voices.txt

echo "<div style=\"position: absolute; top: 110px; left: 20px\">similar voices:</div>" > combined_voices.html
grep -v Final parsed_voices.txt | grep -v "===" >> combined_voices.html
cat "$SCRIPT_DIRECTORY"/scale.html >> combined_voices.html
grep Final parsed_voices.txt > final.txt
