#!/usr/bin/env ruby

#
# Helps to find presentations in television news.
#
class PresentationFinder

  # moderation >= 3 seconds
  MINIMAL_MODERATION_TIME_IN_SECONDS = 3
  # currently not needed because of the next constant

  # minimal number of seconds that have to be in sequence
  MINIMAL_CONSECUTIVE_SECONDS = 4

  # time between moderations >= 15 seconds
  MINIMAL_TIME_BETWEEN_MODERATIONS_IN_SECONDS = 15

  # add an offset at the start and end of a video cut, in this case 2 seconds
  OFFSET_START = 3

  OFFSET_END = 2

  OFFSET_FIRST_PRESENTATION = 3
  OFFSET_LAST_PRESENTATION = 4

  attr_reader :filename, :linesInFile

  def initialize(filename)
    @filename = filename

    # number of lines
    @linesInFile = File.readlines(@filename).size
  end

  def start
    lineCounter = 0
    startSeconds = -1
    endSeconds = -1
    lastSeconds = 0
    consecutiveSeconds = 0
    firstPresentation = 0

    File.open(@filename).each do |line|
      seconds=(line[/seconds: (.*), confidence.*/,1]).to_i
      confidence=(line[/.*, confidence: (.*)/,1]).to_i
      lineCounter += 1

      if startSeconds == -1

        startSeconds = seconds
        lastSeconds = seconds
        consecutiveSeconds = 1

      else

        if (seconds > (lastSeconds + MINIMAL_TIME_BETWEEN_MODERATIONS_IN_SECONDS)) or (lineCounter == @linesInFile)
          if (consecutiveSeconds < MINIMAL_CONSECUTIVE_SECONDS) or ((lastSeconds - startSeconds) < MINIMAL_MODERATION_TIME_IN_SECONDS)
            print "             nothing #{startSeconds} #{lastSeconds}\n"
          else
            if (lineCounter == @linesInFile)
              endSeconds = seconds + OFFSET_LAST_PRESENTATION
            else
              endSeconds = lastSeconds
            end
            if firstPresentation == 0
              firstPresentation = 1
              startSeconds -= OFFSET_FIRST_PRESENTATION
            end

            print "  before: #{startSeconds} - #{endSeconds}, #{consecutiveSeconds}\n"

            startSeconds = startSeconds - OFFSET_START
            endSeconds   = endSeconds   + OFFSET_END
            if startSeconds < 0
              startSeconds = 0
            end

            presentationDuration = endSeconds - startSeconds
            print "  Final: #{startSeconds} - #{endSeconds} = #{presentationDuration}, consecutiveSeconds: #{consecutiveSeconds}\n"

            # add some html
            print "<div style=\"position: absolute; width: #{presentationDuration * 2}px; height:100px; left: #{startSeconds * 2 + 20}px; top: 270px; background: #00ff00; writing-mode: tb-rl; color: white; text-shadow: -2px 2px #000000;\">#{startSeconds}-#{endSeconds}</div>\n";
          end
          startSeconds = seconds
          lastSeconds = seconds
          consecutiveSeconds = 1
        else
          # print "=== #{seconds} #{lastSeconds}\n"
          if seconds == (lastSeconds + 1)
            consecutiveSeconds += 1
          end
          lastSeconds = seconds
        end
      end

      print "#{lineCounter}: #{seconds} #{confidence}\n"

    end

  end

end

myPresentationFinder = PresentationFinder.new(ARGV[0])
myPresentationFinder.start
