#!/usr/bin/env ruby

#
# Helps to combine identified voices.
#
class VoiceCombiner

  # moderation >= 3 seconds
  MINIMAL_MODERATION_TIME_IN_SECONDS = 3
  # currently not needed because of the next constant

  # minimal number of seconds that have to be in sequence
  MINIMAL_CONSECUTIVE_SECONDS = 2

  # time between moderations >= 15 seconds
  MINIMAL_TIME_BETWEEN_MODERATIONS_IN_SECONDS = 15

  # add an offset at the start and end of a video cut, in this case 2 seconds
  OFFSET_START = 4

  OFFSET_END = 2

  OFFSET_FIRST_PRESENTATION = 3
  OFFSET_LAST_PRESENTATION = 4

  COLOR_ARRAY = [ "#1CE6FF", "#FF34FF", "#FF4A46", "#008941", "#006FA6", "#A30059", "#FFDBE5", "#7A4900", "#0000A6", "#63FFAC", "#B79762", "#004D43", "#8FB0FF", "#997D87", "#5A0007", "#809693", "#1B4400", "#4FC601", "#3B5DFF", "#4A3B53", "#FF2F80", "#61615A", "#BA0900", "#6B7900", "#00C2A0", "#FFAA92", "#FF90C9", "#B903AA", "#D16100", "#DDEFFF", "#000035", "#7B4F4B", "#A1C299", "#300018", "#0AA6D8", "#013349", "#00846F", "#372101", "#FFB500", "#C2FFED", "#A079BF", "#CC0744", "#C0B9B2", "#C2FF99", "#001E09", "#00489C", "#6F0062", "#0CBD66", "#EEC3FF", "#456D75", "#B77B68", "#7A87A1", "#788D66", "#885578", "#FAD09F", "#FF8A9A", "#D157A0", "#BEC459", "#456648", "#0086ED", "#886F4C", "#34362D", "#B4A8BD", "#00A6AA", "#452C2C", "#636375", "#A3C8C9", "#FF913F", "#938A81", "#575329", "#00FECF", "#B05B6F", "#8CD0FF", "#3B9700", "#04F757", "#C8A1A1", "#1E6E00", "#7900D7", "#A77500", "#6367A9", "#A05837", "#6B002C", "#772600", "#D790FF", "#9B9700", "#549E79", "#FFF69F", "#201625", "#72418F", "#BC23FF", "#99ADC0", "#3A2465", "#922329", "#5B4534", "#FDE8DC", "#404E55", "#0089A3", "#CB7E98", "#A4E804", "#324E72", "#6A3A4C", "#FFFF00" ]

  attr_reader :filename, :linesInFile, :colorHash, :highestColorNumber

  def initialize(filename)
    @filename = filename

    # number of lines
    @linesInFile = File.readlines(@filename).size

    @colorHash = Hash.new
    @highestColorNumber = 0
  end

  def printFinal(presentationStart, presentationEnd, presentationVoiceIdentifier)
    presentationDuration = presentationEnd - presentationStart
    print "  Final: #{presentationStart} - #{presentationEnd} = #{presentationDuration} , #{presentationVoiceIdentifier}\n"

    if not @colorHash.has_key?(presentationVoiceIdentifier)
      @colorHash[presentationVoiceIdentifier] = COLOR_ARRAY[ highestColorNumber ]

      # print "=== #{@highestColorNumber} #{presentationVoiceIdentifier} #{COLOR_ARRAY[ highestColorNumber ]}\n"

      @highestColorNumber += 1
    end

    print "<div style=\"position: absolute; width: #{presentationDuration * 2}px; height:100px; left: #{presentationStart * 2 + 20}px; top: 140px; background: #{@colorHash[presentationVoiceIdentifier]}; writing-mode: tb-rl; color: white; text-shadow: -2px 2px #000000;\">#{presentationVoiceIdentifier}</div>\n";
  end

  def start
    lineCounter = 0

    presentationStart = -1
    lastSeconds = -1000
    lastVoiceIdentifier = -1

    File.open(@filename).each do |line|
      startSeconds=(line[/^0*(.*) 0.*/,1]).to_i
      duration=(line[/^[^ ]* (0.*) .*/,1]).to_i
      voiceIdentifier=(line[/^[^ ]* [^ ]* S(.*)/,1]).to_i

      lineCounter += 1

      # speaker changed
      if (lastVoiceIdentifier != voiceIdentifier)
        if lastVoiceIdentifier == -1
          presentationStart = startSeconds
          lastSeconds = startSeconds
        else
          presentationEnd = lastSeconds
          presentationVoiceIdentifier = lastVoiceIdentifier
          printFinal(presentationStart, presentationEnd, presentationVoiceIdentifier)
          presentationStart = startSeconds
        end

      # speaker is the same
      else

        # check if presentations of the same speaker belong together
        if lastSeconds < (startSeconds - MINIMAL_TIME_BETWEEN_MODERATIONS_IN_SECONDS)
          presentationEnd = lastSeconds
          presentationVoiceIdentifier = lastVoiceIdentifier
          printFinal(presentationStart, presentationEnd, presentationVoiceIdentifier)
          presentationStart = startSeconds
        end
      end

      # last line => special treatment
      if (lineCounter == @linesInFile)
        presentationEnd = startSeconds + duration
        presentationVoiceIdentifier = voiceIdentifier
        printFinal(presentationStart, presentationEnd, presentationVoiceIdentifier)
      end

      lastVoiceIdentifier = voiceIdentifier
      lastSeconds = startSeconds + duration

      print "=== #{lineCounter}: #{startSeconds} #{duration} #{voiceIdentifier}\n"

    end

  end

end

myVoiceCombiner = VoiceCombiner.new(ARGV[0])
myVoiceCombiner.start
